#!/usr/bin/env python

#############################################################################
##
## Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
## All rights reserved.
## Contact: Nokia Corporation (qt-info@nokia.com)
##
## This file is part of the documentation of Qt. It was originally
## published as part of Qt Quarterly.
##
## $QT_BEGIN_LICENSE:LGPL$
## Commercial Usage
## Licensees holding valid Qt Commercial licenses may use this file in
## accordance with the Qt Commercial License Agreement provided with the
## Software or, alternatively, in accordance with the terms contained in
## a written agreement between you and Nokia.
##
## GNU Lesser General Public License Usage
## Alternatively, this file may be used under the terms of the GNU Lesser
## General Public License version 2.1 as published by the Free Software
## Foundation and appearing in the file LICENSE.LGPL included in the
## packaging of this file.  Please review the following information to
## ensure the GNU Lesser General Public License version 2.1 requirements
## will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
##
## In addition, as a special exception, Nokia gives you certain additional
## rights.  These rights are described in the Nokia Qt LGPL Exception
## version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
##
## GNU General Public License Usage
## Alternatively, this file may be used under the terms of the GNU
## General Public License version 3.0 as published by the Free Software
## Foundation and appearing in the file LICENSE.GPL included in the
## packaging of this file.  Please review the following information to
## ensure the GNU General Public License version 3.0 requirements will be
## met: http://www.gnu.org/copyleft/gpl.html.
##
## If you have questions regarding the use of this file, please contact
## Nokia at qt-info@nokia.com.
## $QT_END_LICENSE$
##
#############################################################################

import sys

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4.QtMobility.QtLocation import *

import simple_mapper_rc

class MapWidget(QGraphicsView):

    positionChanged = pyqtSignal(QGeoCoordinate)
    
    def __init__(self, parent = None):
    
        QGraphicsView.__init__(self, parent)
        self.setCursor(Qt.OpenHandCursor)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        
        self.geoMap = None
        self.dragging = False
        self.dragStartPosition = QPoint()
        self.pressed = False
        
        self.scene = QGraphicsScene()
        self.setScene(self.scene)
        self.setMouseTracking(True)
    
    def setMap(self, geoMap):
    
        self.scene.clear()
        self.scene.addItem(geoMap)
        self.geoMap = geoMap
        self.geoMap.resize(self.scene.sceneRect().width(),
                           self.scene.sceneRect().height())
    
    def mousePressEvent(self, event):
    
        if event.button() == Qt.LeftButton:
        
            self.pressed = True
            self.dragStartPosition = QPoint(event.pos())
            QTimer.singleShot(QApplication.instance().startDragTime(),
                              self.initDrag)
            event.accept()
    
    def initDrag(self):
    
        if self.pressed:
            self.setCursor(Qt.ClosedHandCursor)
            self.dragging = True
    
    def mouseMoveEvent(self, event):
    
        if not self.geoMap:
            return
        
        v = event.pos() - self.dragStartPosition
        
        if self.dragging:
            self.geoMap.pan(-v.x(), -v.y())
            self.dragStartPosition = QPoint(event.pos())
        
        elif self.pressed and v.manhattanLength() >= QApplication.instance().startDragDistance():
            self.setCursor(Qt.ClosedHandCursor)
            self.dragging = True
        
        else:
            self.dragStartPosition = QPoint(event.pos())
            self.positionChanged.emit(self.geoMap.screenPositionToCoordinate(QPointF(event.pos())))
        
        event.accept()
    
    def mouseReleaseEvent(self, event):
    
        self.pressed = False
        
        if self.dragging:
            v = event.pos() - self.dragStartPosition
            self.geoMap.pan(-v.x(), -v.y())
            self.setCursor(Qt.OpenHandCursor)
            self.dragging = False
        
        event.accept()
    
    def resizeEvent(self, event):
    
        if self.geoMap:
            self.scene.setSceneRect(QRectF(0, 0, event.size().width(), event.size().height()))
            self.geoMap.resize(event.size().width(), event.size().height())
        
        QGraphicsView.resizeEvent(self, event)
    
    def wheelEvent(self, event):
    
        steps = event.delta() / 120
        zoom = max(min(self.geoMap.zoomLevel() + steps, self.geoMap.maximumZoomLevel()),
                   self.geoMap.minimumZoomLevel())
        
        if zoom != self.geoMap.zoomLevel():
            self.geoMap.setZoomLevel(zoom)
    
    def mouseDoubleClickEvent(self, event):
    
        self.geoMap.setCenter(self.geoMap.screenPositionToCoordinate(QPointF(event.pos())))


class LocationWidget(QLineEdit):

    def __init__(self, parent = None):
    
        QLineEdit.__init__(self, parent)
        self.setReadOnly(True)
    
    def updateValue(self, coordinate):
    
        self.setText(coordinate.toString())


class MapWindow(QMainWindow):

    def __init__(self):
    
        QMainWindow.__init__(self)
        
        self.mapTypes = {
            QGraphicsGeoMap.StreetMap: self.tr("Street map"),
            QGraphicsGeoMap.SatelliteMapDay: self.tr("Satellite map (day)"),
            QGraphicsGeoMap.SatelliteMapNight: self.tr("Satellite map (night)"),
            QGraphicsGeoMap.TerrainMap: self.tr("Terrain map")
            }
        
        self.mapWidget = MapWidget()
        self.geoMap = None
        
        fileMenu = self.menuBar().addMenu(self.tr("&File"))
        fileMenu.addAction(self.tr("E&xit"), self.close, QKeySequence.Quit)
        
        self.viewMenu = self.menuBar().addMenu(self.tr("&View"))
        
        self.serviceMenu = self.menuBar().addMenu(self.tr("&Services"))
        
        if self.findServices():
            self.setupToolBars()
            self.selectService(self.serviceMenu.actions()[0])
        
        self.setCentralWidget(self.mapWidget)
        self.setWindowTitle(self.tr("Simple Mapper"))
    
    def findServices(self):
    
        services = QGeoServiceProvider.availableServiceProviders()
        
        if not services:
            
            item = self.mapWidget.scene.addText(
                self.tr("Failed to find any map services. Please ensure that "
                        "the location services plugins for Qt Mobility have "
                        "been built and, if necessary, set the QT_PLUGIN_PATH "
                        "environment variable to the location of the Qt Mobility "
                        "plugins directory."))
            
            item.setTextWidth(300)
            self.serviceMenu.setEnabled(False)
            self.viewMenu.setEnabled(False)
            return False
        
        actionGroup = QActionGroup(self)
        actionGroup.setExclusive(True)
        actionGroup.triggered.connect(self.selectService)
        
        managers = []
        
        for name in services:
        
            service = QGeoServiceProvider(name)
            
            if service.error() != QGeoServiceProvider.NoError:
                continue
            
            manager = service.mappingManager()
            
            if manager:
                action = self.serviceMenu.addAction(name)
                action.setCheckable(True)
                action.setData(service)
                actionGroup.addAction(action)
                managers.append(manager)
        
        return actionGroup.actions() != []
    
    def setupToolBars(self):
    
        # Map type controls
        
        mapTypeToolBar = QToolBar(self.tr("Map Type"))
        
        self.mapTypeCombo = QComboBox()
        self.mapTypeCombo.setSizePolicy(QSizePolicy(QSizePolicy.MinimumExpanding,
                                                    QSizePolicy.Fixed))
        self.mapTypeCombo.currentIndexChanged.connect(self.changeMapType)
        
        mapTypeToolBar.addWidget(self.mapTypeCombo)
        
        self.addToolBar(Qt.TopToolBarArea, mapTypeToolBar)
        
        # Zooming controls
        
        zoomToolBar = QToolBar(self.tr("Zoom"))
        
        self.zoomInAction = zoomToolBar.addAction(
            QIcon(QPixmap(":Resources/zoomin.png")), self.tr("Zoom &In"))
        
        self.zoomInAction.setShortcut(QKeySequence.ZoomIn)
        self.viewMenu.addAction(self.zoomInAction)
        
        self.zoomOutAction = zoomToolBar.addAction(
            QIcon(QPixmap(":Resources/zoomout.png")), self.tr("Zoom &Out"))
        self.zoomOutAction.setShortcut(QKeySequence.ZoomOut)
        self.viewMenu.addAction(self.zoomOutAction)
        
        self.zoomInAction.triggered.connect(self.zoomIn)
        self.zoomOutAction.triggered.connect(self.zoomOut)
        
        self.addToolBar(Qt.TopToolBarArea, zoomToolBar)
        
        # Location controls
        
        locationToolBar = QToolBar(self.tr("Location"))
        
        locationWidget = LocationWidget()
        self.mapWidget.positionChanged.connect(locationWidget.updateValue)
        
        locationToolBar.addWidget(locationWidget)
        
        self.addToolBar(Qt.TopToolBarArea, locationToolBar)
    
    def zoomIn(self):
    
        self.updateZoom(self.geoMap.zoomLevel() + 1)
    
    def zoomOut(self):
    
        self.updateZoom(self.geoMap.zoomLevel() - 1)
    
    def updateZoom(self, value):
    
        value = min(max(self.manager.minimumZoomLevel(), value), self.manager.maximumZoomLevel())
        
        self.geoMap.setZoomLevel(value)
        self.zoomInAction.setEnabled(self.geoMap.zoomLevel() < self.manager.maximumZoomLevel())
        self.zoomOutAction.setEnabled(self.geoMap.zoomLevel() > self.manager.minimumZoomLevel())
    
    @pyqtSlot(int)
    def changeMapType(self, index):
    
        self.geoMap.setMapType(self.mapTypeCombo.itemData(index).toInt()[0])
    
    def populateMapTypeCombo(self):
    
        self.mapTypeCombo.clear()
        
        for mapType in self.geoMap.supportedMapTypes():
            try:
                text = self.mapTypes[mapType]
                self.mapTypeCombo.addItem(text, QVariant(mapType))
            except KeyError:
                pass
        
        if self.mapTypeCombo.count() > 0:
            self.geoMap.setMapType(self.mapTypeCombo.itemData(0).toInt()[0])
    
    def selectService(self, action):
    
        action.setChecked(True)
        name = action.text()
        
        if self.geoMap:
            coordinate = self.geoMap.center()
            mapType = self.geoMap.mapType()
            zoomLevel = self.geoMap.zoomLevel()
        else:
            coordinate = QGeoCoordinate(0, 0)
            mapType = QGraphicsGeoMap.StreetMap
            zoomLevel = 1
        
        service = action.data().toPyObject()
        
        self.manager = service.mappingManager()
        self.geoMap = QGraphicsGeoMap(self.manager)
        
        self.geoMap.setCenter(coordinate)
        self.geoMap.setMapType(mapType)
        
        self.populateMapTypeCombo()
        self.updateZoom(zoomLevel)
        
        self.mapWidget.setMap(self.geoMap)
        self.geoMap.zoomLevelChanged.connect(self.updateZoom)


if __name__ == "__main__":

    app = QApplication(sys.argv)
    window = MapWindow()
    window.resize(640, 480)
    window.show()
    sys.exit(app.exec_())
