/****************************************************************************
**
** Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the documentation of Qt. It was originally
** published as part of Qt Quarterly.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QComboBox>
#include <QGeoServiceProvider>
#include <QMenuBar>
#include <QToolBar>

#include "locationwidget.h"
#include "mapwidget.h"
#include "mapwindow.h"

using namespace QtMobility;

MapWindow::MapWindow(QWidget *parent)
    : QMainWindow(parent)
{
    mapTypes[QGraphicsGeoMap::StreetMap] = tr("Street map");
    mapTypes[QGraphicsGeoMap::SatelliteMapDay] = tr("Satellite map (day)");
    mapTypes[QGraphicsGeoMap::SatelliteMapNight] = tr("Satellite map (night)");
    mapTypes[QGraphicsGeoMap::TerrainMap] = tr("Terrain map");

    mapWidget = new MapWidget();
    geoMap = 0;

    QMenu *fileMenu = menuBar()->addMenu(tr("&File"));
    fileMenu->addAction(tr("E&xit"), this, SLOT(close()), QKeySequence::Quit);

    viewMenu = menuBar()->addMenu(tr("&View"));
    serviceMenu = menuBar()->addMenu(tr("&Services"));

    if (findServices()) {
        setupToolBars();
        selectService(serviceMenu->actions()[0]);
    }

    setCentralWidget(mapWidget);
    setWindowTitle(tr("Simple Mapper"));
}

bool MapWindow::findServices()
{
    QStringList services = QGeoServiceProvider::availableServiceProviders();

    if (services.isEmpty()) {

        QGraphicsTextItem *item = mapWidget->scene()->addText(
            tr("Failed to find any map services. Please ensure that "
               "the location services plugins for Qt Mobility have "
               "been built and, if necessary, set the QT_PLUGIN_PATH "
               "environment variable to the location of the Qt Mobility "
               "plugins directory."));

        item->setTextWidth(300);
        serviceMenu->setEnabled(false);
        viewMenu->setEnabled(false);
        return false;
    }

    actionGroup = new QActionGroup(this);
    actionGroup->setExclusive(true);
    connect(actionGroup, SIGNAL(triggered(QAction *)), this, SLOT(selectService(QAction *)));

    foreach (QString name, services) {

        QGeoServiceProvider *service = new QGeoServiceProvider(name);

        if (service->error() != QGeoServiceProvider::NoError) {
            delete service;
            continue;
        }

        QGeoMappingManager *manager = service->mappingManager();

        if (manager) {
            QAction *action = serviceMenu->addAction(name);
            action->setCheckable(true);
            action->setData(managers.count());
            actionGroup->addAction(action);
            managers.append(manager);
        }
    }

    return actionGroup->actions().count() != 0;
}

void MapWindow::setupToolBars()
{
    // Map type controls

    QToolBar *mapTypeToolBar = new QToolBar(tr("Map Type"));

    mapTypeCombo = new QComboBox();
    mapTypeCombo->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding,
                                            QSizePolicy::Fixed));
    connect(mapTypeCombo, SIGNAL(currentIndexChanged(int)),
            this, SLOT(changeMapType(int)));

    mapTypeToolBar->addWidget(mapTypeCombo);

    addToolBar(Qt::TopToolBarArea, mapTypeToolBar);

    // Zooming controls

    QToolBar *zoomToolBar = new QToolBar(tr("Zoom"));

    zoomInAction = zoomToolBar->addAction(
        QIcon(QPixmap(":Resources/zoomin.png")), tr("Zoom &In"));

    zoomInAction->setShortcut(QKeySequence::ZoomIn);
    viewMenu->addAction(zoomInAction);

    zoomOutAction = zoomToolBar->addAction(
        QIcon(QPixmap(":Resources/zoomout.png")), tr("Zoom &Out"));
    zoomOutAction->setShortcut(QKeySequence::ZoomOut);
    viewMenu->addAction(zoomOutAction);

    connect(zoomInAction, SIGNAL(triggered()), this, SLOT(zoomIn()));
    connect(zoomOutAction, SIGNAL(triggered()), this, SLOT(zoomOut()));

    addToolBar(Qt::TopToolBarArea, zoomToolBar);

    // Location controls

    QToolBar *locationToolBar = new QToolBar(tr("Location"));

    LocationWidget *locationWidget = new LocationWidget();
    connect(mapWidget, SIGNAL(positionChanged(const QGeoCoordinate &)),
            locationWidget, SLOT(updateValue(const QGeoCoordinate &)));

    locationToolBar->addWidget(locationWidget);

    addToolBar(Qt::TopToolBarArea, locationToolBar);
}

void MapWindow::zoomIn()
{
    updateZoom(geoMap->zoomLevel() + 1);
}

void MapWindow::zoomOut()
{
    updateZoom(geoMap->zoomLevel() - 1);
}

void MapWindow::updateZoom(qreal value)
{
    value = qBound(manager->minimumZoomLevel(), value, manager->maximumZoomLevel());

    geoMap->setZoomLevel(value);
    zoomInAction->setEnabled(geoMap->zoomLevel() < manager->maximumZoomLevel());
    zoomOutAction->setEnabled(geoMap->zoomLevel() > manager->minimumZoomLevel());
}

void MapWindow::changeMapType(int index)
{
    geoMap->setMapType(QGraphicsGeoMap::MapType(mapTypeCombo->itemData(index).toInt()));
}

void MapWindow::populateMapTypeCombo()
{
    mapTypeCombo->clear();

    foreach (QGraphicsGeoMap::MapType mapType, geoMap->supportedMapTypes()) {
        if (mapTypes.contains(mapType))
            mapTypeCombo->addItem(mapTypes[mapType], QVariant(mapType));
    }

    if (mapTypeCombo->count() > 0)
        geoMap->setMapType(QGraphicsGeoMap::MapType(mapTypeCombo->itemData(0).toInt()));
}

void MapWindow::selectService(QAction *action)
{
    action->setChecked(true);
    QString name = action->text();

    QGeoCoordinate coordinate = QGeoCoordinate(0, 0);
    QGraphicsGeoMap::MapType mapType = QGraphicsGeoMap::StreetMap;
    int zoomLevel = 1;

    if (geoMap) {
        coordinate = geoMap->center();
        mapType = geoMap->mapType();
        zoomLevel = geoMap->zoomLevel();
        geoMap->deleteLater();
    }

    manager = managers[action->data().toInt()];
    geoMap = new QGraphicsGeoMap(manager);

    geoMap->setCenter(coordinate);
    geoMap->setMapType(mapType);

    populateMapTypeCombo();
    updateZoom(zoomLevel);

    mapWidget->setMap(geoMap);
    connect(geoMap, SIGNAL(zoomLevelChanged(qreal)), this, SLOT(updateZoom(qreal)));
}
